package edu.hawaii.contactservice.client;

import org.restlet.ext.xml.DomRepresentation;
import org.restlet.resource.ClientResource;
import org.restlet.resource.ResourceException;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import edu.hawaii.contactservice.common.contact.Contact;


/**
 * A simple example of a "client" class for the DateServer service. 
 * @author Philip Johnson
 */
public class ContactClient {
  /**
   * Connect to a running ContactService and perform the requested operation. 
   * @param args The first arg is the host (such as "http://localhost:8112/").
   * The second arg is the operation, either "put", "get", or "delete". 
   * The third arg is the unique ID of the contact.
   * If the operation is a "put", then there must be three more arguments for 
   * first name, last name, and info. 
   * @throws Exception If problems occur.
   */
  public static void main(String[] args) throws Exception {
    // Make sure we were at least passed the minimum number of arguments. 
    if (args.length < 3) {
      System.out.println("Usage: <host> <operation> <uniqueID> {optional args}");
      System.out.println("Example host: http://localhost:8112");
      return;
    }
    // (1) Figure out the URL from the args and make the ClientResource.
    String host = args[0];
    String contextRoot = "/contactserver/";
    String resource = null;
    String uniqueID = args[2];
    ClientResource client = null;
    
    if ("get-all".equals(args[1]) || "delete-all".equals(args[1])) {
        resource = "contacts";
        String url = host + contextRoot + resource;
        client = new ClientResource(url);
    }
    else {
        resource = "contact/";
        String url = host + contextRoot + resource + uniqueID;
        client = new ClientResource(url);
    }
    
    // (2) Now do the right thing depending on the operation.
    String operation = args[1];
    try {
      if ("get".equals(operation)) {
        DomRepresentation representation = new DomRepresentation(client.get());
        Contact contact = new Contact(representation.getDocument());
        System.out.println(contact.toString());
      }
      else if ("delete".equals(operation)) {
        client.delete();
      }
      else if ("put".equals(operation)) {
        Contact contact = new Contact(args[3], args[4], args[5], args[6], uniqueID);
        DomRepresentation representation = new DomRepresentation();
        representation.setDocument(contact.toXml());
        client.put(representation);
      }
      
      else if ("get-all".equals(operation)) {
          DomRepresentation representation = new DomRepresentation(client.get());
          Document doc = representation.getDocument();
          NodeList nodeList = doc.getElementsByTagName("contact");
          
          // Loop through each <contact> tag and fetch data.
          for (int nodes = 0; nodes < nodeList.getLength(); nodes++) {
             String uri = nodeList.item(nodes).getTextContent();
             DomRepresentation representation2 = 
                 new DomRepresentation(new ClientResource(uri).get());
             Contact contact = new Contact(representation2.getDocument());
             System.out.println(contact.toString());      
          }
        }
        else if ("delete-all".equals(operation)) {
             DomRepresentation representation = new DomRepresentation(client.get());
             Document doc = representation.getDocument();
             NodeList nodeList = doc.getElementsByTagName("contact");
          
             // Loop through each contact and delete contact.
             for (int nodes = 0; nodes < nodeList.getLength(); nodes++) {
                String uri = nodeList.item(nodes).getTextContent();
                ClientResource clientToDelete = new ClientResource(uri);
                clientToDelete.delete();
             }
             System.out.println("Contacts is now empty.");  
        }
    }
    catch (ResourceException e) {
      // If the operation didn't succeed, indicate why here.
      System.out.println("Error: " + e.getStatus());
    }
  }
}
