package edu.hawaii.contactservice.server.resource.contacts;

import java.util.Collection;
import java.util.Iterator;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.restlet.resource.Get;
import org.restlet.resource.ServerResource;
import org.restlet.data.Reference;
import org.restlet.ext.xml.DomRepresentation;
import org.restlet.representation.Representation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import edu.hawaii.contactservice.common.contact.Contact;
import edu.hawaii.contactservice.server.db.Contacts;

/**
 * A server resource that will handle a request for all contacts.
 * Supported representations: XML.
 * @author Emerson Tabucol
 */
public class ContactsResource extends ServerResource {
  
  /**
   * Returns the Contacts instance requested by the URL. 
   * @return The XML representation of all contacts.
   * @throws Exception If problems occur making the representation. Shouldn't occur in 
   * practice but if it does, Restlet will set the Status code. 
   */
  @Get
  public Representation getContacts() throws Exception {
    
    String host = new Reference(getReference(), "").getTargetRef().getHostIdentifier();

    // Create the Document instance representing this XML.
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    DocumentBuilder builder = factory.newDocumentBuilder();
    Document doc = builder.newDocument();
    // Create and attach the root element <contact>.
    Element rootElement = doc.createElement("contacts");
    doc.appendChild(rootElement);
    // Get the singleton instance of the repository.
    Contacts contacts = Contacts.getInstance();
    Collection<Contact> snapshot = contacts.getContacts();
    Iterator<Contact> iter = snapshot.iterator();
    Contact iterContact;
    String id, url;
    while (iter.hasNext()) {
      iterContact = iter.next();
      id = iterContact.getUniqueID();
      url = host + "/contactserver/" + "contact/" + id;
      attachElement(doc, rootElement, "contact", url);
    }
    // Create an empty XML representation.
    DomRepresentation result = new DomRepresentation();
    result.setDocument(doc);
    return result;
  }
  
  /**
   * Helper function that creates a child element and attaches it to the passed parent element.
   * @param doc The document for creating elements. 
   * @param parent The parent element. 
   * @param childName The name of the child element.
   * @param childValue The text value for the child element. 
   */
  private void attachElement(Document doc, Element parent, String childName, String childValue) {
    Element childElement = doc.createElement(childName);
    childElement.setTextContent(childValue);
    parent.appendChild(childElement);
  }
}
